@@ -3,8 +3,9 @@ from __future__ import division
 
 from collections import OrderedDict
 from copy import deepcopy
-import json
 import importlib
+import json
+import warnings
 
 from asteval import Interpreter, get_ast_names, valid_symbol_name
 from numpy import arcsin, array, cos, inf, isclose, nan, sin, sqrt
@@ -74,12 +75,19 @@ class Parameters(OrderedDict):
         for key, val in _syms.items():
             self._asteval.symtable[key] = val
 
-        self.update(*args, **kwds)
-
     def copy(self):
         """Parameters.copy() should always be a deepcopy."""
         return self.__deepcopy__(None)
 
+    def update(self, other):
+        """Update values and symbols with another Parameters object."""
+        if not isinstance(other, Parameters):
+            raise ValueError("'%s' is not a Parameters object" % other)
+        self.add_many(*other.values())
+        for sym in other._asteval.user_defined_symbols():
+            self._asteval.symtable[sym] = other._asteval.symtable[sym]
+        return self
+
     def __copy__(self):
         """Parameters.copy() should always be a deepcopy."""
         return self.__deepcopy__(None)
@@ -136,16 +144,15 @@ class Parameters(OrderedDict):
         if not isinstance(other, Parameters):
             raise ValueError("'%s' is not a Parameters object" % other)
         out = deepcopy(self)
-        params = other.values()
-        out.add_many(*params)
+        out.add_many(*other.values())
+        for sym in other._asteval.user_defined_symbols():
+            if sym not in out._asteval.symtable:
+                out._asteval.symtable[sym] = other._asteval.symtable[sym]
         return out
 
     def __iadd__(self, other):
         """Add/assign Parameters objects."""
-        if not isinstance(other, Parameters):
-            raise ValueError("'%s' is not a Parameters object" % other)
-        params = other.values()
-        self.add_many(*params)
+        self.update(other)
         return self
 
     def __array__(self):
@@ -464,7 +471,7 @@ class Parameters(OrderedDict):
         state = {'unique_symbols': tmp['unique_symbols'],
                  'params': []}
         for parstate in tmp['params']:
-            _par = Parameter()
+            _par = Parameter(name='')
             _par.__setstate__(parstate)
             state['params'].append(_par)
         self.__setstate__(state)
@@ -538,12 +545,12 @@ class Parameter(object):
 
     """
 
-    def __init__(self, name=None, value=None, vary=True, min=-inf, max=inf,
+    def __init__(self, name, value=None, vary=True, min=-inf, max=inf,
                  expr=None, brute_step=None, user_data=None):
         """
         Parameters
         ----------
-        name : str, optional
+        name : str
             Name of the Parameter.
         value : float, optional
             Numerical Parameter value.
@@ -572,7 +579,6 @@ class Parameter(object):
 
         """
         self.name = name
-        self._val = value
         self.user_data = user_data
         self.init_value = value
         self.min = min
@@ -587,6 +593,7 @@ class Parameter(object):
         self.stderr = None
         self.correl = None
         self.from_internal = lambda val: val
+        self._val = value
         self._init_bounds()
 
     def set(self, value=None, vary=None, min=None, max=None, expr=None,
@@ -688,32 +695,31 @@ class Parameter(object):
 
     def __setstate__(self, state):
         """Set state for pickle."""
-        (self.name, self.value, self.vary, self.expr, self.min, self.max,
+        (self.name, _value, self.vary, self.expr, self.min, self.max,
          self.brute_step, self.stderr, self.correl, self.init_value,
          self.user_data) = state
         self._expr_ast = None
         self._expr_eval = None
         self._expr_deps = []
         self._delay_asteval = False
+        self.value = _value
         self._init_bounds()
 
     def __repr__(self):
         """Return printable representation of a Parameter object."""
         s = []
-        if self.name is not None:
-            s.append("'%s'" % self.name)
-        sval = repr(self._getval())
+        sval = "value=%s" % repr(self._getval())
         if not self.vary and self._expr is None:
-            sval = "value=%s (fixed)" % sval
+            sval += " (fixed)"
         elif self.stderr is not None:
-            sval = "value=%s +/- %.3g" % (sval, self.stderr)
+            sval += " +/- %.3g" % self.stderr
         s.append(sval)
         s.append("bounds=[%s:%s]" % (repr(self.min), repr(self.max)))
         if self._expr is not None:
             s.append("expr='%s'" % self.expr)
         if self.brute_step is not None:
             s.append("brute_step=%s" % (self.brute_step))
-        return "<Parameter %s>" % ', '.join(s)
+        return "<Parameter '%s', %s>" % (self.name, ', '.join(s))
 
     def setup_bounds(self):
         """Set up Minuit-style internal/external parameter transformation of
@@ -781,36 +787,27 @@ class Parameter(object):
         """Get value, with bounds applied."""
         # Note assignment to self._val has been changed to self.value
         # The self.value property setter makes sure that the
-        # _expr_eval.symtable is kept updated.
-        # If you just assign to self._val then
-        # _expr_eval.symtable[self.name]
+        # _expr_eval.symtable is kept up-to-date.
+        # If you just assign to self._val then _expr_eval.symtable[self.name]
         # becomes stale if parameter.expr is not None.
         if (isinstance(self._val, uncertainties.core.Variable) and
                 self._val is not nan):
-
+            msg = ("Please make sure that the Parameter value is a number, "
+                   "not an instance of 'uncertainties.core.Variable'. This "
+                   "automatic conversion will be removed in the next release.")
+            warnings.warn(FutureWarning(msg))
             try:
                 self.value = self._val.nominal_value
             except AttributeError:
                 pass
-        if not self.vary and self._expr is None:
-            return self._val
 
         if self._expr is not None:
             if self._expr_ast is None:
                 self.__set_expression(self._expr)
-
             if self._expr_eval is not None:
                 if not self._delay_asteval:
                     self.value = self._expr_eval(self._expr_ast)
                     check_ast_errors(self._expr_eval)
-
-        if self._val is not None:
-            if self._val > self.max:
-                self._val = self.max
-            elif self._val < self.min:
-                self._val = self.min
-        if self._expr_eval is not None:
-            self._expr_eval.symtable[self.name] = self._val
         return self._val
 
     def set_expr_eval(self, evaluator):
@@ -826,10 +823,15 @@ class Parameter(object):
     def value(self, val):
         """Set the numerical Parameter value."""
         self._val = val
+        if self._val is not None:
+            if self._val > self.max:
+                self._val = self.max
+            elif self._val < self.min:
+                self._val = self.min
         if not hasattr(self, '_expr_eval'):
             self._expr_eval = None
         if self._expr_eval is not None:
-            self._expr_eval.symtable[self.name] = val
+            self._expr_eval.symtable[self.name] = self._val
 
     @property
     def expr(self):
@@ -884,9 +886,10 @@ class Parameter(object):
         """positive"""
         return +self._getval()
 
-    def __nonzero__(self):
-        """not zero"""
+    def __bool__(self):
+        """bool"""
         return self._getval() != 0
+    __nonzero__ = __bool__  # TODO: remove when PY3 only
 
     def __int__(self):
         """int"""
@@ -908,10 +911,10 @@ class Parameter(object):
         """-"""
         return self._getval() - other
 
-    def __div__(self, other):
+    def __truediv__(self, other):
         """/"""
         return self._getval() / other
-    __truediv__ = __div__
+    __div__ = __truediv__  # TODO: remove when PY3 only
 
     def __floordiv__(self, other):
         """//"""
@@ -961,10 +964,10 @@ class Parameter(object):
         """+ (right)"""
         return other + self._getval()
 
-    def __rdiv__(self, other):
+    def __rtruediv__(self, other):
         """/ (right)"""
         return other / self._getval()
-    __rtruediv__ = __rdiv__
+    __rdiv__ = __rtruediv__  # TODO: remove when PY3 only
 
     def __rdivmod__(self, other):
         """divmod (right)"""
@@ -993,5 +996,7 @@ class Parameter(object):
 
 def isParameter(x):
     """Test for Parameter-ness."""
+    msg = 'The isParameter function will be removed in the next release.'
+    warnings.warn(FutureWarning(msg))
     return (isinstance(x, Parameter) or
             x.__class__.__name__ == 'Parameter')
