@@ -1,3 +1,5 @@
+import functools
+import sys
 import unittest
 import warnings
 
@@ -24,6 +26,22 @@ def _skip_if_no_pandas():
         raise pytest.skip("Skipping tests that require pandas.")
 
 
+def firstarg_ndarray(func):
+    """a simple wrapper used for testing that wrapped
+    functions can be model functions"""
+    @functools.wraps(func)
+    def wrapper(x, *args, **kws):
+        x = np.asarray(x)
+        return func(x, *args, **kws)
+    return wrapper
+
+
+@firstarg_ndarray
+def linear_func(x, a, b):
+    "test wrapped model function"
+    return a*x+b
+
+
 class CommonTests(object):
     # to be subclassed for testing predefined models
 
@@ -35,14 +53,14 @@ class CommonTests(object):
             args = self.args
         except AttributeError:
             self.model = self.model_constructor()
-            self.model_drop = self.model_constructor(missing='drop')
-            self.model_raise = self.model_constructor(missing='raise')
+            self.model_omit = self.model_constructor(nan_policy='omit')
+            self.model_raise = self.model_constructor(nan_policy='raise')
             self.model_explicit_var = self.model_constructor(['x'])
             func = self.model.func
         else:
             self.model = self.model_constructor(*args)
-            self.model_drop = self.model_constructor(*args, missing='drop')
-            self.model_raise = self.model_constructor(*args, missing='raise')
+            self.model_omit = self.model_constructor(*args, nan_policy='omit')
+            self.model_raise = self.model_constructor(*args, nan_policy='raise')
             self.model_explicit_var = self.model_constructor(
                 *args, independent_vars=['x'])
             func = self.model.func
@@ -93,7 +111,7 @@ class CommonTests(object):
             val2 = out2.values[parname]
             if max_diff < abs(val1-val2):
                 max_diff = abs(val1-val2)
-        assert(max_diff > 1.e-8)
+        assert max_diff > 1.e-8
 
     def test_result_attributes(self):
         pars = self.model.make_params(**self.guess())
@@ -142,12 +160,12 @@ class CommonTests(object):
         pars = self.model.make_params(**self.guess())
         result = self.model.fit(self.data, pars, x=self.x)
         report = result.fit_report()
-        assert("[[Model]]" in report)
-        assert("[[Variables]]" in report)
-        assert("[[Fit Statistics]]" in report)
-        assert(" # function evals   =" in report)
-        assert(" Akaike " in report)
-        assert(" chi-square " in report)
+        assert "[[Model]]" in report
+        assert "[[Variables]]" in report
+        assert "[[Fit Statistics]]" in report
+        assert " # function evals   =" in report
+        assert " Akaike " in report
+        assert " chi-square " in report
 
     def test_data_alignment(self):
         _skip_if_no_pandas()
@@ -165,7 +183,7 @@ class CommonTests(object):
 
         # Skip over missing (NaN) values, aligning via pandas index.
         data.iloc[500:510] = np.nan
-        result = self.model_drop.fit(data, params, x=x)
+        result = self.model_omit.fit(data, params, x=x)
         assert_results_close(result.values, self.true_values())
 
         # Raise if any NaN values are present.
@@ -216,6 +234,9 @@ class CommonTests(object):
 class TestUserDefiniedModel(CommonTests, unittest.TestCase):
     # mainly aimed at checking that the API does what it says it does
     # and raises the right exceptions or warnings when things are not right
+    import six
+    if six.PY2:
+        from six import assertRaisesRegex
 
     def setUp(self):
         self.true_values = lambda: dict(amplitude=7.1, center=1.1, sigma=2.40)
@@ -443,7 +464,7 @@ class TestUserDefiniedModel(CommonTests, unittest.TestCase):
         self.assertTrue(abs(result.params['bkg_c'].value - 1.0) < 0.25)
 
         comps = mod.eval_components(x=self.x)
-        assert('bkg_' in comps)
+        assert 'bkg_' in comps
 
     def test_composite_has_bestvalues(self):
         # test that a composite model has non-empty best_values
@@ -453,6 +474,42 @@ class TestUserDefiniedModel(CommonTests, unittest.TestCase):
         mod = model1 + model2
         pars = mod.make_params()
 
+        values1 = dict(amplitude=7.10, center=1.1, sigma=2.40)
+        values2 = dict(amplitude=12.2, center=2.5, sigma=0.5)
+        data = (gaussian(x=self.x, **values1) + gaussian(x=self.x, **values2)
+                + 0.1*self.noise)
+
+        pars['g1_sigma'].set(value=2)
+        pars['g1_center'].set(value=1, max=1.5)
+        pars['g1_amplitude'].set(value=3)
+        pars['g2_sigma'].set(value=1)
+        pars['g2_center'].set(value=2.6, min=2.0)
+        pars['g2_amplitude'].set(value=1)
+
+        result = mod.fit(data, params=pars, x=self.x)
+
+        self.assertTrue(len(result.best_values) == 6)
+
+        self.assertTrue(abs(result.params['g1_amplitude'].value - 7.1) < 0.5)
+        self.assertTrue(abs(result.params['g2_amplitude'].value - 12.2) < 0.5)
+        self.assertTrue(abs(result.params['g1_center'].value - 1.1) < 0.2)
+        self.assertTrue(abs(result.params['g2_center'].value - 2.5) < 0.2)
+
+        for name, par in pars.items():
+            assert len(repr(par)) > 5
+
+    def test_composite_plotting(self):
+        # test that a composite model has non-empty best_values
+        pytest.importorskip("matplotlib")
+        import matplotlib
+        matplotlib.use('Agg')
+
+        model1 = models.GaussianModel(prefix='g1_')
+        model2 = models.GaussianModel(prefix='g2_')
+
+        mod = model1 + model2
+        pars = mod.make_params()
+
         values1 = dict(amplitude=7.10, center=1.1, sigma=2.40)
         values2 = dict(amplitude=12.2, center=2.5, sigma=0.5)
         data = (gaussian(x=self.x, **values1) + gaussian(x=self.x, **values2)
@@ -466,13 +523,14 @@ class TestUserDefiniedModel(CommonTests, unittest.TestCase):
         pars['g2_amplitude'].set(1)
 
         result = mod.fit(data, params=pars, x=self.x)
+        fig, ax = result.plot(show_init=True)
 
-        self.assertTrue(len(result.best_values) == 6)
+        assert isinstance(fig, matplotlib.figure.Figure)
+        assert isinstance(ax, matplotlib.axes.GridSpec)
 
-        self.assertTrue(abs(result.params['g1_amplitude'].value - 7.1) < 0.5)
-        self.assertTrue(abs(result.params['g2_amplitude'].value - 12.2) < 0.5)
-        self.assertTrue(abs(result.params['g1_center'].value - 1.1) < 0.2)
-        self.assertTrue(abs(result.params['g2_center'].value - 2.5) < 0.2)
+        comps = result.eval_components(x=self.x)
+        assert len(comps) == 2
+        assert 'g1_' in comps
 
     def test_hints_in_composite_models(self):
         # test propagation of hints from base models to composite model
@@ -567,6 +625,7 @@ class TestUserDefiniedModel(CommonTests, unittest.TestCase):
         self.assertTrue(result.params['pos_delta'].value > 0)
 
     def test_model_nan_policy(self):
+        """Tests for nan_policy with NaN values in the input data."""
         x = np.linspace(0, 10, 201)
         np.random.seed(0)
         y = gaussian(x, 10.0, 6.15, 0.8)
@@ -574,14 +633,18 @@ class TestUserDefiniedModel(CommonTests, unittest.TestCase):
         y += gaussian(x, 0.25, 6.00, 7.5)
         y += np.random.normal(size=len(x), scale=0.5)
 
+        # with NaN values in the input data
         y[55] = y[91] = np.nan
         mod = PseudoVoigtModel()
         params = mod.make_params(amplitude=20, center=5.5,
                                  sigma=1, fraction=0.25)
         params['fraction'].vary = False
+
         # with raise, should get a ValueError
         result = lambda: mod.fit(y, params, x=x, nan_policy='raise')
-        self.assertRaises(ValueError, result)
+        msg = ('NaN values detected in your input data or the output of your '
+               'objective/model function - fitting algorithms cannot handle this!')
+        self.assertRaisesRegex(ValueError, msg, result)
 
         # with propagate, should get no error, but bad results
         result = mod.fit(y, params, x=x, nan_policy='propagate')
@@ -602,6 +665,52 @@ class TestUserDefiniedModel(CommonTests, unittest.TestCase):
         self.assertTrue(abs(result.params['amplitude'].value - 20.0) < 5.0)
         self.assertTrue(abs(result.params['center'].value - 6.0) < 0.5)
 
+        # with 'wrong_argument', should get a ValueError
+        err_msg = r"nan_policy must be 'propagate', 'omit', or 'raise'."
+        with pytest.raises(ValueError, match=err_msg):
+            mod.fit(y, params, x=x, nan_policy='wrong_argument')
+
+    def test_model_nan_policy_NaNs_by_model(self):
+        """Test for nan_policy with NaN values generated by the model function."""
+        def double_exp(x, a1, t1, a2, t2):
+            return a1*np.exp(-x/t1) + a2*np.exp(-(x-0.1) / t2)
+
+        model = Model(double_exp)
+
+        truths = (3.0, 2.0, -5.0, 10.0)
+        x = np.linspace(1, 10, 250)
+        np.random.seed(0)
+        y = double_exp(x, *truths) + 0.1*np.random.randn(x.size)
+
+        p = model.make_params(a1=4, t1=3, a2=4, t2=3)
+        result = lambda: model.fit(data=y, params=p, x=x, method='Nelder',
+                                   nan_policy='raise')
+
+        msg = 'The model function generated NaN values and the fit aborted!'
+        self.assertRaisesRegex(ValueError, msg, result)
+
+    @pytest.mark.skipif(sys.version_info.major == 2,
+                        reason="cannot use wrapped functions with Python 2")
+    def test_wrapped_model_func(self):
+        x = np.linspace(-1, 1, 51)
+        y = 2.0*x + 3 + 0.0003 * x*x
+        y += np.random.normal(size=len(x), scale=0.025)
+        mod = Model(linear_func)
+        pars = mod.make_params(a=1.5, b=2.5)
+
+        tmp = mod.eval(pars, x=x)
+
+        self.assertTrue(tmp.max() > 3)
+        self.assertTrue(tmp.min() > -20)
+
+        result = mod.fit(y, pars, x=x)
+        self.assertTrue(result.chisqr < 0.05)
+        self.assertTrue(result.aic < -350)
+        self.assertTrue(result.errorbars)
+
+        self.assertTrue(abs(result.params['a'].value - 2.0) < 0.05)
+        self.assertTrue(abs(result.params['b'].value - 3.0) < 0.41)
+
 
 class TestLinear(CommonTests, unittest.TestCase):
 
